% Perform calculation of the Scattering Coefficients (Mie Series) 
% for the n-layered sphere. Max number of Coeff denoted by nmax 
% 
%       [a_n,b_n] = nlayerScaCoeff(m,x,nmax,nlayers);
%       x is the vector of size parameter (k.*radius) per compartment relating to
%       the vector of relative refractive indices m


% G. Chliveros and MA Rodrigues (2002), CVPRAI Group, Sheffield Hallam University
% http://www.shu.ac.uk/scis/artificial_intelligence
function [a_n,b_n] = nlayerScaCoeff(m,x,nmax,nlayers)

m = m(:).';
x = x(:).';
nlayers = length(x);
if length(x)==1
   x = x*ones(size(m));
end
if length(m)>1 & length(x)~=length(m)
   error('Dimensions of x & m must be the same or scalar')
end
N = ((1:nmax).')*ones(1,length(x));

% Ricatti-Bessel functions
psi = RB1(x, nmax);
psim = RB1(m.*x, nmax);
% Ricatti-Bessel function for variable m(k+1)*x(k) of k number of layers
psimKplus1 = zeros(nmax,length(x));
for k = 1:(nlayers-1)
    psimK = RB1(m(k+1)*x(k), nmax);
    psimKplus1(:,k+1) = psimK;
    k=k+1;
end
correctpsimn_1 = [0, sin(m(2:nlayers).*x(1:(nlayers-1)))];
% and for (n-1) series terms Ricatti-Bessel
psimKplus1n_1 = [correctpsimn_1; psimKplus1(1:(nmax-1),:)];
% Ricatti-Bessel functions
kappa = RB2(x, nmax);
kappam = RB2(m.*x, nmax);
% Ricatti-Bessel function for variable m(k+1)*x(k) of k number of layers
kappamKplus1 = zeros(nmax,length(x));
for k = 1:(nlayers-1)
    kappamK = RB1(m(k+1)*x(k), nmax);
    kappamKplus1(:,k+1) = kappamK;
    k=k+1;
end
correctkappan_1 = [0, -cos(m(2:nlayers).*x(1:(nlayers-1)))];
kappamKplus1n_1 = [correctkappan_1;kappamKplus1(1:(nmax-1),:)]; % (n-1) terms
% Hankel function
xi = psi - i * kappa;
% Ricatti-Bessel function for variable m(k+1)*x(k) of k number of layers, for (n-1) terms in series
psin_1 = [sin(x);psi(1:(nmax-1),:)];
psimn_1 = [sin(m.*x);psim(1:(nmax-1),:)];
kappan_1 = [-cos(x);kappa(1:(nmax-1),:)];
kappamn_1 = [-cos(m.*x);kappam(1:(nmax-1),:)];

% correction to avoid error in matrix/array dimensions
if length(m)>1
   mm = ones(nmax,1)*m;
end
if length(x)>1
   xx = ones(nmax,1)*x;
end
% Calculation of the first derivative of the nth order R-B function
% Refer to Bohren and Huffman (1998), Wiley, pp86-87
dpsi = psin_1-N.*psi./xx;
dpsim = psimn_1-N.*psim./(mm.*xx);
correctionA = [ones(1,nmax); (mm(:,2:nlayers).*xx(:,1:(nlayers - 1))).'].';
dpsimKplus1 = psimKplus1n_1 - N.*psimKplus1./correctionA;
dkappa = kappan_1-N.*kappa./xx;
dkappam = kappamn_1-N.*kappam./(mm.*xx);
dkappamKplus1 = kappamKplus1n_1 - N.*kappamKplus1./correctionA;
dxi = dpsi - i * dkappa;

% Main routine. Reference: Volkov and Kovach(1990), Izvestiya Atmospheric Oceanic Physics, 26(5), 381-385
A_k = zeros(nmax,1);
B_k = zeros(nmax,1);
for k = 1:(nlayers-1)
    a_k = m(k).*kappam(:,k).*dpsimKplus1(:,k+1) - m(k+1).*dkappam(:,k).*psimKplus1(:,k+1);
    dash_a_k =  m(k).*kappam(:,k).*dkappamKplus1(:,k+1) - m(k+1).*dkappam(:,k).*kappamKplus1(:,k+1);
    b_k = m(k).*dkappam(:,k).*psimKplus1(:,k+1) - m(k+1).*kappam(:,k).*dpsimKplus1(:,k+1);
    dash_b_k = m(k).*dkappam(:,k).*dkappamKplus1(:,k+1) - m(k+1).*kappam(k).*dkappamKplus1(:,k+1);
    numA_k  = (m(k+1).*psimKplus1(:,k+1).*dpsim(k) - m(k).*dpsimKplus1(:,k+1).*psim(:,k) + A_k.*a_k);
    denomA_k = (m(k+1).*kappamKplus1(:,k+1).*dpsim(:,k) - m(k).*dkappamKplus1(:,k+1).*psim(:,k) + A_k .*dash_a_k);
    A_k =  numA_k ./ denomA_k;
    numB_k = (m(k+1).*psim(:,k).*dpsimKplus1(:,k+1) - m(k).*psimKplus1(:,k+1).*dpsim(:,k) + B_k.*b_k);
    denomB_k = (m(k+1).*psim(:,k).*dkappamKplus1(:,k+1) - m(k).*dpsim(:,k).*kappamKplus1(:,k+1) + B_k.*dash_b_k);
    B_k =  numB_k ./ denomB_k;
end

NUMa_n = psi(:,nlayers) .* (dpsim(:,nlayers) - A_k .* dkappam(:,nlayers)) - m(nlayers).*dpsi(:,nlayers) .* (psim(:,nlayers) - A_k.*kappam(:,nlayers));
DENOMa_n = xi(:,nlayers) .* (dpsim(:,nlayers) - A_k.*dkappam(:,nlayers)) - m(nlayers).*dxi(:,nlayers) .* (psi(:,nlayers) - A_k.*kappam(:,nlayers));

NUMb_n = m(nlayers).*psi(:,nlayers) .* (dpsim(:,nlayers) - B_k .* dkappam(:,nlayers)) - dpsi(:,nlayers) .* (psim(:,nlayers) - B_k.*kappam(:,nlayers));
DENOMb_n = m(nlayers).*xi(:,nlayers) .* (dpsim(:,nlayers) - B_k.*dkappam(:,nlayers)) - dxi(:,nlayers) .* (psi(:,nlayers) - B_k.*kappam(:,nlayers));

% the coefficients...
a_n = NUMa_n ./ DENOMa_n;
b_n = NUMb_n ./ DENOMb_n;